@@ -9,11 +9,8 @@ dbFileConnect <- function(dbfile)
     ## files (for our use case, the .sqlite file _must_ exist):
     if (!file.exists(dbfile))
         stop("DB file '", dbfile, "' not found")
-    ## We should not need to explicitly library(RSQLite) because it's in
-    ## Depends and Imports but this seems to make 'R CMD check hgu95av2.db' happier.
-    library(RSQLite)
-    dbConnect(SQLite(), dbname=dbfile, cache_size=64000, synchronous="off",
-              flags=SQLITE_RO)
+    dbConnect(SQLite(), dbname=dbfile, cache_size=64000, 
+              synchronous="off", flags=SQLITE_RO)
 }
 
 ### Used at unload time (in .onUnload).
@@ -22,10 +19,6 @@ dbFileDisconnect <- function(dbconn)
     dbDisconnect(dbconn)
 }
 
-
-
-    
-
 ### Used at load time (in .onLoad) by the SQLite-based ann data package to
 ### dynamically add exported symbols to its namespace environment.
 
@@ -39,8 +32,6 @@ dbFileDisconnect <- function(dbconn)
     }
 }
 
-
-
 addToNamespaceAndExport <- function(x, value, pkgname)
 {
     prefix <- sub('.db','',pkgname)    
@@ -87,7 +78,6 @@ addToNamespaceAndExport <- function(x, value, pkgname)
     namespaceExport(ns, x)
 }
 
-
 mergeToNamespaceAndExport <- function(envir, pkgname)
 {
     keys <- ls(envir, all.names=TRUE)
