@@ -58,66 +58,10 @@ createAnnObjs <- function(class, seeds, seed0, envir=NULL)
 
 ## Function to get all these create statements out of the namespace.
 createAnnObjs.SchemaChoice = function(schema, prefix, target, dbconn, datacache){
-    switch(schema,
-           "ORGANISM_DB" = return(AnnotationDbi:::createAnnObjs.ORGANISM_DB(prefix, target, dbconn, datacache)),
-
-           "HUMANCROSSCHIP_DB" = return(AnnotationDbi:::createAnnObjs.HUMANCROSSCHIP_DB(prefix, target, dbconn, datacache)),
-           
-           "ANOPHELESCHIP_DB" = return(AnnotationDbi:::createAnnObjs.ANOPHELESCHIP_DB(prefix, target, dbconn, datacache)),
-           "ARABIDOPSISCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.ARABIDOPSISCHIP_DB(prefix, target, dbconn, datacache)),
-           "BOVINECHIP_DB"  = return(AnnotationDbi:::createAnnObjs.BOVINECHIP_DB(prefix, target, dbconn, datacache)),
-           "CANINECHIP_DB"  = return(AnnotationDbi:::createAnnObjs.CANINECHIP_DB(prefix, target, dbconn, datacache)),
-           "CHICKENCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.CHICKENCHIP_DB(prefix, target, dbconn, datacache)),
-           "CHIMPCHIP_DB" = return(AnnotationDbi:::createAnnObjs.CHIMPCHIP_DB(prefix, target, dbconn, datacache)),
-           "COELICOLORCHIP_DB" = return(AnnotationDbi:::createAnnObjs.COELICOLORCHIP_DB(prefix, target, dbconn, datacache)),
-           "ECOLICHIP_DB"  = return(AnnotationDbi:::createAnnObjs.ECOLICHIP_DB(prefix, target, dbconn, datacache)),
-           "FLYCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.FLYCHIP_DB(prefix, target, dbconn, datacache)),
-           "HUMANCHIP_DB" = return(AnnotationDbi:::createAnnObjs.HUMANCHIP_DB(prefix, target, dbconn, datacache)),
-           "MALARIACHIP_DB" = return(AnnotationDbi:::createAnnObjs.MALARIACHIP_DB(prefix, target, dbconn, datacache)),
-           "MOUSECHIP_DB"  = return(AnnotationDbi:::createAnnObjs.MOUSECHIP_DB(prefix, target, dbconn, datacache)),
-           "PIGCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.PIGCHIP_DB(prefix, target, dbconn, datacache)),
-           "RATCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.RATCHIP_DB(prefix, target, dbconn, datacache)),
-           "RHESUSCHIP_DB" = return(AnnotationDbi:::createAnnObjs.RHESUSCHIP_DB(prefix, target, dbconn, datacache)),
-           "WORMCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.WORMCHIP_DB(prefix, target, dbconn, datacache)),
-           "XENOPUSCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.XENOPUSCHIP_DB(prefix, target, dbconn, datacache)),
-           "YEASTCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.YEASTCHIP_DB(prefix, target, dbconn, datacache)),
-           "ZEBRAFISHCHIP_DB"  = return(AnnotationDbi:::createAnnObjs.ZEBRAFISHCHIP_DB(prefix, target, dbconn, datacache)),
-           
-           "ANOPHELES_DB"  = return(AnnotationDbi:::createAnnObjs.ANOPHELES_DB(prefix, target, dbconn, datacache)),
-           "ARABIDOPSIS_DB"  = return(AnnotationDbi:::createAnnObjs.ARABIDOPSIS_DB(prefix, target, dbconn, datacache)),
-           "BOVINE_DB"  = return(AnnotationDbi:::createAnnObjs.BOVINE_DB(prefix, target, dbconn, datacache)),
-           "CANINE_DB"  = return(AnnotationDbi:::createAnnObjs.CANINE_DB(prefix, target, dbconn, datacache)),
-           "CHICKEN_DB"  = return(AnnotationDbi:::createAnnObjs.CHICKEN_DB(prefix, target, dbconn, datacache)),
-           "CHIMP_DB"  = return(AnnotationDbi:::createAnnObjs.CHIMP_DB(prefix, target, dbconn, datacache)),
-           "COELICOLOR_DB"  = return(AnnotationDbi:::createAnnObjs.COELICOLOR_DB(prefix, target, dbconn, datacache)),
-           "ECOLI_DB"  = return(AnnotationDbi:::createAnnObjs.ECOLI_DB(prefix, target, dbconn, datacache)),
-           "FLY_DB"  = return(AnnotationDbi:::createAnnObjs.FLY_DB(prefix, target, dbconn, datacache)),
-           "HUMAN_DB"  = return(AnnotationDbi:::createAnnObjs.HUMAN_DB(prefix, target, dbconn, datacache)),           
-           "MALARIA_DB"  = return(AnnotationDbi:::createAnnObjs.MALARIA_DB(prefix, target, dbconn, datacache)),
-           "MOUSE_DB"  = return(AnnotationDbi:::createAnnObjs.MOUSE_DB(prefix, target, dbconn, datacache)),
-           "PIG_DB"  = return(AnnotationDbi:::createAnnObjs.PIG_DB(prefix, target, dbconn, datacache)),
-           "RAT_DB"  = return(AnnotationDbi:::createAnnObjs.RAT_DB(prefix, target, dbconn, datacache)),
-           "RHESUS_DB"  = return(AnnotationDbi:::createAnnObjs.RHESUS_DB(prefix, target, dbconn, datacache)),
-           "WORM_DB"  = return(AnnotationDbi:::createAnnObjs.WORM_DB(prefix, target, dbconn, datacache)),
-           "XENOPUS_DB"  = return(AnnotationDbi:::createAnnObjs.XENOPUS_DB(prefix, target, dbconn, datacache)),
-           "YEAST_DB"  = return(AnnotationDbi:::createAnnObjs.YEAST_DB(prefix, target, dbconn, datacache)),
-           "ZEBRAFISH_DB"  = return(AnnotationDbi:::createAnnObjs.ZEBRAFISH_DB(prefix, target, dbconn, datacache)),
-           
-           "GO_DB"  = return(AnnotationDbi:::createAnnObjs.GO_DB(prefix, target, dbconn, datacache)),
-
-           "KEGG_DB"  = return(AnnotationDbi:::createAnnObjs.KEGG_DB(prefix, target, dbconn, datacache)),
-
-           "INPARANOID_DB"  = return(AnnotationDbi:::createAnnObjs.INPARANOID_DB(prefix, target, dbconn, datacache)),
-           
-           "PFAM_DB"  = return(AnnotationDbi:::createAnnObjs.PFAM_DB(prefix, target, dbconn, datacache)),
-           
-           "AFFYHUEX_DB"  = return(AnnotationDbi:::createAnnObjs.AFFYHUEX_DB(prefix, target, dbconn, datacache))
-          )
+    FUN <- get(paste0("createAnnObjs.", schema))
+    FUN(prefix, target, dbconn, datacache)
 }
 
-
-
-
 createAnnDbBimap <- function(seed, seed0)
 {
     for (slot in names(seed0)) {
@@ -225,6 +169,17 @@ makeSeedList <- function(species, fields)
 ## Below are helper functions to define helpful startup messages for
 ## especially ornery packages:
 
+## TODO: for these organsism add a waring about Blast2GO (being stale)
+## taxIDList = c("Canis_familiaris"="9615",
+##                 "Sus_Scrofa"="9823",
+##                 "Anopheles_gambiae"="180454",
+##                 "Xenopus_laevis"="8355",
+##                 "Macaca_mulatta"="9544",
+##                 "Pan_troglodytes"="9598")
+## ##                 "Escherichia_coli_K12"="511145")
+
+
+
 choosePackage <- function(pkgType){
 ## defines a blackList to choose the type of message based on the pkg name.
 ## If you are not in the blackList, then we don't need a message.
@@ -260,6 +215,16 @@ choosePackage <- function(pkgType){
                  ## Pass given on 11/8/10
                  ## "hs25kresogen.db" = "deprecatedSimonDeBernard",
                  ## "mm24kresogen.db" = "deprecatedSimonDeBernard",
+                 "org.Cf.eg" = "B2GOstale",
+                 "org.Ss.eg" = "B2GOstale",
+                 "org.Ag.eg" = "B2GOstale",
+                 "org.Xl.eg" = "B2GOstale",
+                 "org.Mmu.eg" = "B2GOstale",
+                 "org.Pt.eg" = "B2GOstale",
+                 "canine.db" = "B2GOstale",
+                 "canine2.db" = "B2GOstale",
+                 "xlaevis.db" = "B2GOstale",
+                 ## End of the B2GO warnings (remove once problem is resolved)
                  "org.Sco.eg.db" = "deprecated",
                  "KEGG.db" = "keggstale",
                  "NO_MESSAGE_TYPE"
@@ -300,13 +265,20 @@ annotMessage <- function(msgType, pkgType){
 		  "illumina_XXX_ARRAYADDRESS mapping to convert between", 
 		  "BeadIDs and Illumina IDs. \n\n  *_XXX_ here refers to the",
   		  "species and version for this platform"),
-                "keggstale" = paste("\n",pkgType,"contains mappings based on older",
+                "keggstale" = paste("\n",pkgType,"contains mappings based on",
+                  " older",
                   "data because the original resource was removed from the",
                   "the public domain before the most recent update was",
                   "produced. This package should now be considered deprecated",
                   "and future versions of Bioconductor may not have it",
                   "available.  Users who want more current data are encouraged",
                   "to look at the KEGGREST or reactome.db packages"),
+                "B2GOstale" = paste("\n",pkgType,"contains GO mappings based",
+                  "on older data",
+                  "because the Blast2GO data resource was removed from",
+                  "the public domain just before the most recent update was",
+                  "produced. We are working on an alternative means to get",
+                  "this kind of data before the next release."),
                 "exon_probeset" = paste("\n",pkgType,"is based on exon",
                   "probesets. For a more gene-centric view, use the",
                   "transcriptcluster version of this package."),
